'''
this module contains printing functions for definitions form various sources
each source (like google and urban dictionary) needs to have it's own printing format
due to the vriations in the information
'''

import os

import pyfiglet

from urb.wrappers import google, urban


def print_urb(definition: urban.UrbanDefinition):
    '''
    the printing function for Urban Dictionary definitions
    prameter is a UrbanDefinition object returned from the API wrapper
    '''
    print(pyfiglet.figlet_format(definition.word, font='cybermedium'))

    def format_def(_def):
        if any([str(i) in _def.strip()[:3] for i in range(10)]):
            _def = '• ' + _def.strip()[3:]
        return _def
    def format_eg(_eg):
        _eg = _eg.strip()
        truth = [str(i) in _eg.strip()[:3] for i in range(10)]
        if any(truth):
            truth_index = _eg.index(str(truth.index(True)))
            while _eg[truth_index] != ' ':
                truth_index += 1
            _eg = _eg[truth_index:].strip()
        return _eg

    print(
        '\n'.join(
            list(
                map(format_def, definition.definition.split('\n'))
            )
        )
    )
    # print(f"{definition.definition}")

    lines = definition.example.split('\n')
    if 'example' in lines[0].lower():
        definition.example = '\n'.join(lines[1:])
    print('\nexample ─\n')
    print(
        '\n'.join(
            map(format_eg, lines)
        )
    )
    # print(f"\nexample - \n\n{definition.example}")

    try:
        print(f"\nauthor - {definition.author} | 👍 {definition.thumbs_up} | 👎 {definition.thumbs_down} :)")
    except AttributeError:
        print(f"\nauthor - {definition.author} :)")


def print_goog(definition: google.GoogleDefinition, index: int = 0):
    '''
    the printing formatter for definitions from the google dictionary api
    parameter is a GoogleDefinition object returned fromt the API wrapper
    '''
    print(pyfiglet.figlet_format(definition.word, font='cybermedium').strip(), '\n')

    if definition.phonetic is not None:
        print(definition.phonetic)

    if definition.origin is not None:
        print(f"\norigin - \n{definition.origin}\n")

    if index > len(definition.meanings):
        index = 0
    if index != 0:
        index += 1
        definition.meanings = [definition.meanings[:index - 1][-1]]
    for meaning in definition.meanings:
        print(meaning.part_of_speech)
        print((len(meaning.definition) + int((os.get_terminal_size().columns - len(meaning.definition)) / 2)) * "─")
        print(meaning.definition)
        if meaning.example is not None:
            print(meaning.example, "\n")



"""
the cli logic managing all the commands, arguments and options

currently implemented commands are

define
    usage -
        urb define <search_term>
        e.g. urb define hello

    args -
        --google, -g
        uses the google dictionary API instead of urban dictionary
        e.g. urb define --google hello
            urb define -g hello

        --index, -i
        APIs teend to provide multiple definitions for a word at time.
        this option lets specify what definition is to be displayed
        e.g. urb define -i 2 noob
             urb define --index boon
"""

import click

from urb.wrappers import google, urban
from urb import prints


@click.group()
@click.option('--index', '-i', 'index', type=bool, is_flag=True,help='specific definition to display')
@click.option('--google', '-g', 'goog', type=bool, is_flag=True, help='use google search instead.')
def urb(goog, index): #pylint: disable=unused-argument
    # the group definition.
    # empty cuz idk what i can do with it
    pass

@urb.command("define")
@click.option('--google', '-g', 'goog', type=bool, default=False, is_flag=True)
@click.option('--index', '-i', 'index', type=int, default=0, required=False)
@click.argument("word")
def define(word, goog, index):
    '''
    the define command.
    usage and options specified at the beginning of the file
    '''
    try:
        if goog:
            definition = google.define(word)
            prints.print_goog(definition[0], index)
        else:
            definition = urban.define(word)
            prints.print_urb(definition[:index + 1][-1])
    except ConnectionError:
        print('please connect to the internet :)')
    except IndexError:
        print('\nno definitions found for this word :(')

def main():
    urb(obj={})

if __name__ == "__main__":
    main()



import pathlib
from setuptools import setup, find_packages

HERE = pathlib.Path(__file__).parent

README = (HERE / "README.md").read_text()

setup(
    name="urb",
    version="0.1.3",
    description="a command line dictionary and wrapper for various dictionary APIs",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/addyett/urb",
    author="addyett",
    author_email="g.aditya2048@gmail.com",
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Operating System :: MacOS",
        "Operating System :: Microsoft :: Windows :: Windows 10",
        "Operating System :: POSIX :: Linux",
    ],
    packages=find_packages(),
    include_package_data=True,
    install_requires=["pyfiglet", "requests", "click"],
    entry_points={
        "console_scripts": [
            "urb=urb.__main__:main",
        ]
    },
)



"""
just a sneazy wrapper around the Free Dictionary API from https://dictionaryapi.dev/
allows defining searching for a definition for a word
"""

from typing import List
import requests

FD_ENTRIES_URL = 'https://api.dictionaryapi.dev/api/v2/entries/en/'

class DefinitionMeaning():

    __slots__ = (
        'part_of_speech',
        'definition',
        'example',
        'synonyms',
        'antonyms'
    )

    def __init__(self, **data) -> None:
        self.part_of_speech = data.get('partOfSpeech', None)
        self.definition = data.get('definitions')[0]['definition']
        self.example = data.get('definitions')[0].get('example', None)
        self.synonyms = data.get('definitions')[0]['synonyms']
        self.antonyms = data.get('definitions')[0]['antonyms']


class GoogleDefinition():

    __slots__ = (
        'word',
        'phonetic',
        'origin',
        'meanings'
    )

    def __init__(self, **data) -> None:
        self.word = data.pop('word', None)
        self.phonetic = data.pop('phonetic', None)
        self.origin = data.pop('origin', None)
        self.meanings = [DefinitionMeaning(**meaning) for meaning in data.pop('meanings', [])]

    def __repr__(self):
        return f"<GoogleDefinition {self.word!r}>"
    def __str__(self):
        return str(self.word)

def _get_google_json(url: str) -> dict:
    try:
        data = requests.get(url).json()
    except Exception as exception:
        print()
        data = []
    return data

def _parse_google_json(data):
    results = []
    if len(data) <= 0 or 'resolution' in data:
        data = []
        return data
    for _definition in data:
        results.append(GoogleDefinition(**_definition))
    return results

def define(term) -> List[GoogleDefinition]:
    '''
    returns definition for a word

    parameters:
        term: str

    e.g. urban.define('hello')
    '''
    data = _get_google_json(FD_ENTRIES_URL + term)
    return _parse_google_json(data)

if __name__ == '__main__':
    definition = define('lmao')



"""
yet anotehr wrapper around the Urban Dictionary API
provides methods to get results for queried words as well as random words
"""

from typing import List
import requests

UD_DEFINE_URL = 'https://api.urbandictionary.com/v0/define?term='
UD_RANDOM_URL = 'https://api.urbandictionary.com/v0/random'

class UrbanDefinition():

    __slot__ = (
        'word',
        'definition',
        'example',
        'author',
        'thumbs_up',
        'thumbs_down',
        'id'
    )

    def __init__(self, **data) -> None:
        self.word = data.pop('word', None)
        self.definition = data.pop('definition', None)
        self.example = data.pop('example', None)
        self.author = data.pop('author', None)
        self.thumbs_up = data.pop('thumbs_up', None)
        self.thumbs_down  = data.pop('thumbs_down', None)
        self.id = data.pop('defid', None)

    def __repr__(self):
        return f"<UrbanDefinition {self.word!r} - {self.id!r}>"

    def __str__(self):
        return str(self.word)

def _get_urban_json(url: str) -> dict:
    return requests.get(url).json()

def _parse_urban_json(json_response):
    results = []
    # print(json_response)
    # [print(key, ': ', json_response['list'][0][key]) for key in json_response['list'][0].keys()]
    if json_response is None or any([e in json_response for e in ('ERROR', 'error')]):
        raise ValueError("Urb: invalid request")
    if 'list' not in json_response or len(json_response['list']) < 1:
        return results
    for definition in json_response['list']:
        results.append(UrbanDefinition(**definition))
    return results

def define(term) -> List[UrbanDefinition]:
    '''
    returns definitions for a word

    parameters:
        term: str

    e.g. urban.define('hello')
    '''
    json_response = _get_urban_json(UD_DEFINE_URL + term)
    return _parse_urban_json(json_response)

def random() -> List[UrbanDefinition]:
    '''
    returns definitions of 10 random words
    '''
    json_response = _get_urban_json(UD_RANDOM_URL)
    return _parse_urban_json(json_response)

if __name__ == '__main__':
    rando = random()
